/********************************************************************/
/* COPYRIGHTS:                                                      */
/*                                                                  */
/*   This module contains code made available by IBM Corporation on */
/* an AS IS basis.  Any one receiving the module is  considered  to */
/* be  licensed under IBM copyrights to use the IBM-provided source */
/* code in any way he or  she  deems  fit,  including  copying  it, */
/* compiling  it,  modifying  it,  and  redistributing  it, with or */
/* without modifications.  No license  under  any  IBM  patents  or */
/* patent  applications  is  to  be  implied  from  this  copyright */
/* license.                                                         */
/*                                                                  */
/*   A user of the module should understand that IBM cannot provide */
/* technical support for the module and will not be responsible for */
/* any consequences of use of the program.                          */
/*                                                                  */
/*   Any notice,  including this one, are not to  be  removed  from */
/* the module with out the prior written consent of IBM.            */
/*                                                                  */
/********************************************************************/

#include "windows.h"
#include "tchar.h"
#include "stdio.h"
#include "iostream.h"

#include "cwbrc.h"
#include "cwbsv.h"


#define RMTCMDB_OK      0
#define RMTCMDB_WARNING 1
#define RMTCMDB_ERROR   4

#define CHECKRC(Msg)	\
  {						\
    if (rc != CWB_OK)	\
    {					\
      printRC(Msg, rc); \
      return(RMTCMDB_ERROR); \
    }\
  };


//Forward declares
void	printUsage();
void	printCAErrors(LPCTSTR title, const unsigned int rc, const cwbSV_ErrHandle h);
void	printRC(LPCTSTR title, const unsigned int rc);		
int		connect();
void	disconnect();

//Globals
cwbSV_ErrHandle hErr_	= 0;
cwbRC_SysHandle hRCSys_	= 0;
cwbCO_SysHandle hCOSys_	= 0;
LPTSTR szRequest_		= NULL; 
LPTSTR szSystem_		= NULL; 
LPTSTR szUid_			= NULL;	
LPTSTR szPwd_			= NULL;


///////////////////////////////////////////////////////////////////////////////
//main
///////////////////////////////////////////////////////////////////////////////
void main(int argc, char* argv[])
{		

	//USES_CONVERSION;	//include the ATL string conversion macros
	
	///////////////////////////////
	//Parse the command line args
	if ( argc != 5 )
	{ 
		printUsage(); 
		exit(RMTCMDB_ERROR);
	}
	szRequest_		= argv[1]; 
	szSystem_		= argv[2]; 
	szUid_			= argv[3];	
	szPwd_			= argv[4];
	unsigned int rc	= 0;
	int  errLevel	= RMTCMDB_OK;

	///////////////////////////////
	//Connect
	errLevel = connect();
	if (errLevel != RMTCMDB_OK)
	{
		disconnect();
		exit( errLevel );
	}

	///////////////////////////////
	//Run the command
	rc = cwbRC_RunCmd(hRCSys_,
					szRequest_,
					hErr_);
	if (rc == CWB_OK)
	{
		printf("Command ran successfully");
	}
	else
	{
		printCAErrors(_T("Run Command"), rc, hErr_);
		errLevel = RMTCMDB_ERROR;
	}
	
	//////////////////////////////
	//Cleanup
	disconnect();

	exit( errLevel );		//Use exit to set ERRORLEVEL 
}

///////////////////////////////////////////////////////////////////////////////
//displayUsage()
///////////////////////////////////////////////////////////////////////////////
void printUsage()
{
	printf("\n\n");
	printf("IBM iSeries Support Center sample for unattended rmtcmd.\n");
	printf("   This sample program wraps the iSeries Access cwbRC APIs.\n");
	printf("   The sample accepts a user ID and password in addition \n");
	printf("   to the command to run and system. \n");
	printf("   Because the prompt mode is set to \"never\" it can  \n");
	printf("   be used in unattended environments such as with \n");
	printf("   incoming remote command service (IRC) \n");
	printf("   or the Windows scheduler service.\n");
	printf("   \n");
	printf("   This sample is provided on an AS IS basis.\n");
	printf("   A user of this sample should understand that IBM cannot provide\n");
	printf("   technical support for the module and will not be responsible for\n");
	printf("   any consequences of use of the program.\n");
	printf("   Comments concerning this sample can be sent to ddilling@us.ibm.com.\n");
	printf("\n");
	printf("Syntax:\n\n");		
	
	printf("RMTCMDB  command system userID password\n");
	printf("         command - Command to execute on the iSeries.\n");
	printf("                   Must be enclosed in double quotes if the command \n");
	printf("                   contains spaces. \n"); 
	printf("         system  - A valid iSeries system name or ip address. \n");
	printf("         userID  - A valid iSeries user profile for the system \n");
	printf("                   specified in the request.\n");
	printf("         password - A valid password for the specified user profile.\n\n");
	printf("All parameters are required!\n\n");
	return;
}

///////////////////////////////////////////////////////////////////////////////
//connect
///////////////////////////////////////////////////////////////////////////////
int connect()
{
	unsigned int rc = 0;
	
	// Initialize
	rc = cwbSV_CreateErrHandle(&hErr_);
	CHECKRC(_T("Connection attempt failed"));

	//Connect
	rc = cwbCO_CreateSystem( szSystem_, &hCOSys_);
	CHECKRC(_T("Connection attempt failed"));

	rc = cwbCO_SetUserIDEx(hCOSys_, szUid_);
	CHECKRC(_T("Connection attempt failed"));
	rc = cwbCO_SetPassword(hCOSys_, szPwd_);
	CHECKRC(_T("Connection attempt failed"));
	rc = cwbCO_SetPromptMode(hCOSys_, CWBCO_PROMPT_NEVER);
	CHECKRC(_T("Connection attempt failed"));
	// Start the system 
	rc = cwbRC_StartSysEx(hCOSys_, &hRCSys_);
	CHECKRC(_T("Connection attempt failed"));

	return RMTCMDB_OK;
}	//end connect


///////////////////////////////////////////////////////////////////////////////
//disconnect
///////////////////////////////////////////////////////////////////////////////
void disconnect()
{
	unsigned int rc = 0;
	if (hErr_ != 0)
		rc = cwbSV_DeleteErrHandle(hErr_);
	hErr_ = 0;	

	if (hRCSys_ != 0)
		rc = cwbRC_StopSys(hRCSys_);
	hRCSys_ = 0;

	if (hCOSys_ != 0)
		rc = cwbCO_DeleteSystem(hCOSys_);
	hCOSys_ = 0;
}	//end disconnect()

///////////////////////////////////////////////////////////////////////////////
//printCAErrors
///////////////////////////////////////////////////////////////////////////////
void printCAErrors(LPCTSTR					title, 
				   const unsigned int		rc, 
				   const cwbSV_ErrHandle	h)
{
	// Get the number of errors   
	unsigned long errorCount = 0;
	if (h == 0 ||											//No error handle
		CWB_OK != cwbSV_GetErrCount( h, &errorCount) ||		//Can't get a count
		errorCount == 0)									//No errors to get
	{
		printRC(title, rc);
		return;
	}

	//Print out all the errors.  Most recent first.
	printf("%s failed with return code %d\n", title, rc);
	TCHAR szErrorText[256];
	for (int i = errorCount; i > 0; i-- )
	{		
		unsigned long returnedLength = 0;
		/* Get the error text and print it out */
		unsigned int rc = cwbSV_GetErrTextIndexed(h,
											i,
											szErrorText,
											sizeof(szErrorText),
											&returnedLength);
		printf("%s\n", szErrorText);		
	}
	return;
}	//printCAErrors

////////////////////////////////////////////////////////////////////
//
void	printRC(LPCTSTR title, const unsigned int rc)
{
	TCHAR  msg[512] = _T("\0");
	switch (rc)
	{
		case CWB_COMMUNICATIONS_ERROR:
			_tcscpy(msg, _T("A communications error occurred."));
			break;
		case CWB_INVALID_POINTER: 
			_tcscpy(msg, _T("Bad or NULL pointer."));
			break;
		case CWBRC_INVALID_SYSTEM_HANDLE: 
			_tcscpy(msg, _T("Invalid system handle."));
			break;
		case CWBRC_REJECTED_USER_EXIT: 
			_tcscpy(msg, _T("Command rejected by user exit program."));
			break;
		//case CWBRC_USR_EXIT_ERROR: 
		//	_tcscpy(msg, _T("Error in user exit program."));
		//	break;
		case CWBRC_COMMAND_FAILED: 
			_tcscpy(msg, _T("Command failed."));
			break;
		case CWBRC_COMMAND_TOO_LONG: 
			_tcscpy(msg, _T("Command string is too long."));
			break;
		case CWB_NOT_ENOUGH_MEMORY: 
			_tcscpy(msg, _T("Insufficient memory."));
			break;
		case CWB_NON_REPRESENTABLE_UNICODE_CHAR: 
			_tcscpy(msg, _T("One or more input UNICODE characters have no representation in the codepage being used. "));
			break;
		case CWB_API_ERROR: 
			_tcscpy(msg, _T("General API failure."));
			break;

		//create system		
		case  CWB_INVALID_SYSNAME:
			_tcscpy(msg, _T("The system name is not valid."));
			break;
		case CWB_RESTRICTED_BY_POLICY:
			_tcscpy(msg, _T("A policy exists that prohibits the user from running this command."));
			break;

		//Start
		case    CWB_SERVER_PROGRAM_NOT_FOUND:
			_tcscpy(msg, _T("AS/400 application not found."));
			break;
		case    CWB_HOST_NOT_FOUND:
			_tcscpy(msg, _T("AS/400 system inactive or does not exist."));
			break;
		case    CWB_SECURITY_ERROR:
			_tcscpy(msg, _T("A security error has occurred."));
			break;
		case    CWB_LICENSE_ERROR:
			_tcscpy(msg, _T("A license error has occurred."));
			break;
		case    CWB_CONFIG_ERROR:
			_tcscpy(msg, _T("A configuration error has occurred."));
			break;
		case    CWBRC_SYSTEM_NAME:
			_tcscpy(msg, _T("System name is too long."));
			break;

		default:
			_tcscpy(msg, _T("Unknown return code."));
	}	//end switch
	printf("%s failed with return code %d: ""%s""\n", title, rc, msg);
	return;
}	//end printRC()